// Copyright (c) 2015 HigPen Works (Kenichiro Matsuura / Yuki Tsukasa). All rights reserved.
#include "Shader.h"

static const GLfloat position_array[]={-1, 1, 0, -1, -1, 0, 1, 1, 0, 1, -1, 0};
static const GLfloat uv_array[]={0, 0, 0, 1, 1, 0, 1, 1};

GLuint CShader::Compile(GLenum type, NSString* file) {
	GLuint shader;
	GLint status;
	const GLchar* source=(GLchar*)[[NSString stringWithContentsOfFile:file encoding:NSUTF8StringEncoding error:nil] UTF8String];
	if (!source) return NSLog(@"Failed to load vertex shader"), false;
	shader=glCreateShader(type);
	glShaderSource(shader, 1, &source, 0);
	glCompileShader(shader);
#if defined(DEBUG)
	GLint log_length;
	glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &log_length);
	if (log_length>0) {
		GLchar* log=(GLchar*)malloc(log_length);
		glGetShaderInfoLog(shader, log_length, &log_length, log);
		NSLog(@"Shader compile log:\n%s", log);
		free(log);
	}
#endif
	glGetShaderiv(shader, GL_COMPILE_STATUS, &status);
	if (status==0) glDeleteShader(shader), shader=0;
	return shader;
}

bool CShader::Link() {
	GLint status;
	glLinkProgram(Program);
#if defined(DEBUG)
	GLint log_length;
	glGetProgramiv(Program, GL_INFO_LOG_LENGTH, &log_length);
	if (log_length>0) {
		GLchar* log = (GLchar*)malloc(log_length);
		glGetProgramInfoLog(Program, log_length, &log_length, log);
		NSLog(@"Program link log:\n%s", log);
		free(log);
	}
#endif
	glGetProgramiv(Program, GL_LINK_STATUS, &status);
	return status!=0;
}

bool CShader::Validate() {
	GLint log_length, status;
	glValidateProgram(Program);
	glGetProgramiv(Program, GL_INFO_LOG_LENGTH, &log_length);
	if (log_length>0) {
		GLchar* log=(GLchar*)malloc(log_length);
		glGetProgramInfoLog(Program, log_length, &log_length, log);
		NSLog(@"Program validate log:\n%s", log);
		free(log);
	}
	glGetProgramiv(Program, GL_VALIDATE_STATUS, &status);
	return status!=0;
}

CShader::CShader() : Program(0) {
	Program=glCreateProgram();
	
	NSString* vspath=[[NSBundle mainBundle] pathForResource:@"Shader" ofType:@"vsh"];
	GLuint vs=Compile(GL_VERTEX_SHADER, vspath);
	if (vs) glAttachShader(Program, vs); else NSLog(@"Failed to compile vertex shader");

	NSString* fspath=[[NSBundle mainBundle] pathForResource:@"Shader" ofType:@"fsh"];
	GLuint fs=Compile(GL_FRAGMENT_SHADER, fspath);
	if (fs) glAttachShader(Program, fs); else NSLog(@"Failed to compile fragment shader");

	glBindAttribLocation(Program, 0, "position");
	glBindAttribLocation(Program, 1, "uv");
	if (Link()) {
		UMatrix=glGetUniformLocation(Program, "matrix");
		UColor=glGetUniformLocation(Program, "color");
		UTexture=glGetUniformLocation(Program, "texture");
	} else {
		NSLog(@"Failed to link program: %d", Program);
	}
	
	if (vs) glDetachShader(Program, vs), glDeleteShader(vs);
	if (fs) glDetachShader(Program, fs), glDeleteShader(fs);
}

CShader::~CShader() {
	if (Program) glDeleteProgram(Program);
}

void CShader::Begin() {
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glDisable(GL_CULL_FACE);
	glDisable(GL_DEPTH_TEST);
	glUseProgram(Program);
	glVertexAttribPointer(0, 3, GL_FLOAT, 0, 0, position_array);
	glVertexAttribPointer(1, 2, GL_FLOAT, 0, 0, uv_array);
	glEnableVertexAttribArray(0);
	glEnableVertexAttribArray(1);
	glActiveTexture(GL_TEXTURE0);
	glUniform1i(UTexture, 0);
	Validate();
}

void CShader::Draw(CTexture* texture, GLKMatrix4 matrix, CColor color) {
	if (!texture) return;
	glUniformMatrix4fv(UMatrix, 1, 0, matrix.m);
	glUniform4fv(UColor, 1, (GLfloat*)&color);
	glBindTexture(GL_TEXTURE_2D, texture->Texture.name);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
}

void CShader::End() {
}

